/*
 *  Copyright (c) 2018-2024 Cloudera, Inc. All rights reserved.
 *
 *  This code is provided to you pursuant to your written agreement with Cloudera, which may be the terms of the
 *  Affero General Public License version 3 (AGPLv3), or pursuant to a written agreement with a third party authorized
 *  to distribute this code.  If you do not have a written agreement with Cloudera or with an authorized and
 *  properly licensed third party, you do not have any rights to this code.
 *
 *  If this code is provided to you under the terms of the AGPLv3:
 *   (A) CLOUDERA PROVIDES THIS CODE TO YOU WITHOUT WARRANTIES OF ANY KIND;
 *   (B) CLOUDERA DISCLAIMS ANY AND ALL EXPRESS AND IMPLIED WARRANTIES WITH RESPECT TO THIS CODE, INCLUDING BUT NOT
 *       LIMITED TO IMPLIED WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE;
 *   (C) CLOUDERA IS NOT LIABLE TO YOU, AND WILL NOT DEFEND, INDEMNIFY, OR HOLD YOU HARMLESS FOR ANY CLAIMS ARISING
 *       FROM OR RELATED TO THE CODE; AND
 *   (D) WITH RESPECT TO YOUR EXERCISE OF ANY RIGHTS GRANTED TO YOU FOR THE CODE, CLOUDERA IS NOT LIABLE FOR ANY
 *       DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED
 *       TO, DAMAGES RELATED TO LOST REVENUE, LOST PROFITS, LOSS OF INCOME, LOSS OF BUSINESS ADVANTAGE OR
 *       UNAVAILABILITY, OR LOSS OR CORRUPTION OF DATA.
 *
 */
package com.cloudera.naaf.aws.lambda;

import com.cloudera.cdp.shaded.org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;
import java.util.stream.Collectors;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class StatelessNiFiFunctionHandlerTest {

    private PropertiesReader propertiesReader;

    @BeforeEach
    public void init() {
        propertiesReader = mock(PropertiesReader.class);
        when(propertiesReader.getProperty("nifi.version")).thenReturn("1.0.0");
    }

    @Test
    public void testEnableAwsSecretsManager() throws IOException {
        final Map<String, String> properties = new TreeMap<>();
        final Map<String, String> environmentVariables = new HashMap<>();
        environmentVariables.put("PARAM_CONTEXT_Kafka", "KafkaSecret, CommonSecret ");
        environmentVariables.put("PARAM_CONTEXT_Elasticsearch", "ElasticsearchSecret, CommonSecret ");

        StatelessNiFiFunctionHandler.enableAwsSecretsManager(properties, environmentVariables, propertiesReader);

        final String propertiesText = properties.entrySet().stream()
                .map(entry -> String.format("%s=%s", entry.getKey(), entry.getValue()))
                .collect(Collectors.joining(System.getProperty("line.separator")));

        final String expectedResult = IOUtils.toString(StatelessNiFiFunctionHandlerTest.class.getClassLoader()
                .getResourceAsStream("aws-secrets-expected.properties"));
        assertEquals(expectedResult, propertiesText);
    }
}
