/*
 * (c) 2018-2021 Cloudera, Inc. All rights reserved.
 *
 *   This code is provided to you pursuant to your written agreement with Cloudera, which may be the terms of the
 *  Affero General Public License version 3 (AGPLv3), or pursuant to a written agreement with a third party authorized
 *  to distribute this code.  If you do not have a written agreement with Cloudera or with an authorized and
 *  properly licensed third party, you do not have any rights to this code.
 *
 *   If this code is provided to you under the terms of the AGPLv3:
 *   (A) CLOUDERA PROVIDES THIS CODE TO YOU WITHOUT WARRANTIES OF ANY KIND;
 *   (B) CLOUDERA DISCLAIMS ANY AND ALL EXPRESS AND IMPLIED WARRANTIES WITH RESPECT TO THIS CODE, INCLUDING BUT NOT
 *       LIMITED TO IMPLIED WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE;
 *   (C) CLOUDERA IS NOT LIABLE TO YOU, AND WILL NOT DEFEND, INDEMNIFY, OR HOLD YOU HARMLESS FOR ANY CLAIMS ARISING
 *       FROM OR RELATED TO THE CODE; AND
 *   (D) WITH RESPECT TO YOUR EXERCISE OF ANY RIGHTS GRANTED TO YOU FOR THE CODE, CLOUDERA IS NOT LIABLE FOR ANY
 *       DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED
 *       TO, DAMAGES RELATED TO LOST REVENUE, LOST PROFITS, LOSS OF INCOME, LOSS OF BUSINESS ADVANTAGE OR
 *       UNAVAILABILITY, OR LOSS OR CORRUPTION OF DATA.
 *
 */

package com.cloudera.naaf.azure.functions;

import com.cloudera.naaf.StatelessNiFiUtil;
import com.microsoft.azure.functions.ExecutionContext;
import com.microsoft.azure.functions.annotation.EventGridTrigger;
import com.microsoft.azure.functions.annotation.FunctionName;
import org.apache.nifi.stateless.flow.TriggerResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.concurrent.atomic.AtomicReference;

public class StatelessNiFiEventGridTriggerFunction {
    private static final Logger logger = LoggerFactory.getLogger(StatelessNiFiEventGridTriggerFunction.class);

    private static final AtomicReference<StatelessNiFiAzureFunctionHolder> functionHolderReference = new AtomicReference<>();

    private StatelessNiFiAzureFunctionHolder getFunctionHolder() {
        StatelessNiFiAzureFunctionHolder functionHolder = functionHolderReference.get();
        if (functionHolder == null) {
            logger.debug("Function has not been initialized yet, initializing now...");
            try {
                functionHolder = new StatelessNiFiAzureFunctionHolder();
            } catch (final IOException e) {
                logger.error("Error initializing function", e);
                throw new RuntimeException(e);
            }
            functionHolderReference.set(functionHolder);
            logger.debug("Successfully initialized Trigger Function");

            return functionHolder;
        }

        logger.debug("Function is already initialized");
        return functionHolder;
    }

    @FunctionName("StatelessNiFiEventGridTriggerFunction")
    public void run( @EventGridTrigger(name = "event", dataType = "string") final String event, final ExecutionContext context) {
        final StatelessNiFiAzureFunctionHolder functionHolder = getFunctionHolder();
        final String instanceId = functionHolder.getFunctionName();
        final TriggerResult triggerResult = functionHolder.getStatelessNiFiFunction()
                .trigger(instanceId, new ByteArrayInputStream(event.getBytes(StandardCharsets.UTF_8)), StatelessNiFiAzureFunctionHolder.getAttributes(context));

        StatelessNiFiUtil.handleTriggerResult(triggerResult);
    }

    @Override
    protected void finalize() {
        functionHolderReference.get().getStatelessNiFiFunction().cleanUp();
    }
}
