/*
 * (c) 2018-2021 Cloudera, Inc. All rights reserved.
 *
 *   This code is provided to you pursuant to your written agreement with Cloudera, which may be the terms of the
 *  Affero General Public License version 3 (AGPLv3), or pursuant to a written agreement with a third party authorized
 *  to distribute this code.  If you do not have a written agreement with Cloudera or with an authorized and
 *  properly licensed third party, you do not have any rights to this code.
 *
 *   If this code is provided to you under the terms of the AGPLv3:
 *   (A) CLOUDERA PROVIDES THIS CODE TO YOU WITHOUT WARRANTIES OF ANY KIND;
 *   (B) CLOUDERA DISCLAIMS ANY AND ALL EXPRESS AND IMPLIED WARRANTIES WITH RESPECT TO THIS CODE, INCLUDING BUT NOT
 *       LIMITED TO IMPLIED WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE;
 *   (C) CLOUDERA IS NOT LIABLE TO YOU, AND WILL NOT DEFEND, INDEMNIFY, OR HOLD YOU HARMLESS FOR ANY CLAIMS ARISING
 *       FROM OR RELATED TO THE CODE; AND
 *   (D) WITH RESPECT TO YOUR EXERCISE OF ANY RIGHTS GRANTED TO YOU FOR THE CODE, CLOUDERA IS NOT LIABLE FOR ANY
 *       DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED
 *       TO, DAMAGES RELATED TO LOST REVENUE, LOST PROFITS, LOSS OF INCOME, LOSS OF BUSINESS ADVANTAGE OR
 *       UNAVAILABILITY, OR LOSS OR CORRUPTION OF DATA.
 *
 */

package com.cloudera.naaf;

import org.apache.nifi.flowfile.FlowFile;
import org.apache.nifi.stateless.flow.TriggerResult;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Map;

public interface StatelessNiFiFunction {
    File TEMP_DIR = new File(System.getProperty("java.io.tmpdir"));

    // Environment Variables
    String INPUT_PORT_NAME_VAR = "INPUT_PORT";
    String OUTPUT_PORT_NAME_VAR = "OUTPUT_PORT";
    String FAILURE_PORTS_VAR = "FAILURE_PORTS";
    String DFX_BASE_URL_VAR = "DF_SERVICE_URL";
    String FLOW_VERSION_CRN_VAR = "FLOW_CRN";
    String PRIVATE_KEY_VAR = "DF_PRIVATE_KEY";
    String ACCESS_KEY_VAR = "DF_ACCESS_KEY";
    String SENSITIVE_PROPS_VAR = "SENSITIVE_PROPS_KEY";
    String NEXUS_BASE_URL_VAR = "NEXUS_URL";
    String PARAM_CONTEXT_MAPPING_PREFIX = "PARAM_CONTEXT_";
    String DEFAULT_PARAMETER_CONTEXT_VAR = "DEFAULT_PARAM_CONTEXT";
    String CONTENT_REPO_LOCATION_VAR = "CONTENT_REPO";
    String DISABLE_STATE_PROVIDER = "DISABLE_STATE_PROVIDER";
    String EXTENSIONS_DOWNLOAD_DIR = "EXTENSIONS_DOWNLOAD_DIR";
    String WORKING_DIR = "WORKING_DIR";
    String EXTENSIONS_DIR_PREFIX = "EXTENSIONS_DIR_";
    String KRB5_CONF_FILE = "KRB5_FILE";
    String PROCESSOR_START_TIMEOUT = "PROCESSOR_START_TIMEOUT";
    String COMPONENT_ENABLE_TIMEOUT = "COMPONENT_ENABLE_TIMEOUT";

    String DEFAULT_KRB5_CONF_FILE = "/etc/krb5.conf";

    // Environment Variables for internal testing.  TODO: Remove before publishing NaaF
    String FLOW_URL_OVERRIDE = "FLOW_URL_OVERRIDE"; // Bypasses DF download and pulls dataflow directly from a URL

    String CLOUD_PROVIDER_ATTRIBUTE = "cloud.provider";

    /**
     * Triggers a NiFi Stateless data flow.
     * @param instanceId The unique identifier for a deployed instance of the function
     * @param inputStream The function input stream.  If null, the dataflow will receive an empty flowfile
     * @param inputAttributes Optional input attributes
     * @return The Stateless trigger result
     */
    TriggerResult trigger(String instanceId, InputStream inputStream, Map<String, String> inputAttributes);

    /**
     * Triggers a NiFi Stateless data flow, writing any successful result to an output stream.
     * @param instanceId The unique identifier for a deployed instance of the function
     * @param inputStream The function input stream.  If null, the dataflow will receive an empty flowfile
     * @param inputAttributes Optional input attributes
     * @param outputStream The function output
     * @throws IOException In case of I/O error
     */
    void trigger(String instanceId, InputStream inputStream, Map<String, String> inputAttributes, OutputStream outputStream) throws IOException;

    /**
     * Returns the output flowfile from the trigger result.  If there is not exactly one result, null is returned.
     * @param triggerResult A trigger result
     * @return A flowfile from the configured output port, or null if not exactly one flowfile is returned.
     */
    FlowFile getOutputFlowfile(TriggerResult triggerResult);

    /**
     * Performs any resource cleanup.
     */
    void cleanUp();
}
