/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.cloudera.naaf.resources;

import java.io.File;
import java.nio.file.Path;
import java.nio.file.Paths;

public abstract class AbstractResourceDownloader implements ResourceDownloader {

    /**
     * Returns the path to which the file should be downloaded.
     * @param objectName An object name in a cloud bucket.
     * @param downloadDirectory The download directory
     * @return The download path, with parent directories created
     */
    protected Path getDownloadPath(final String objectName, final File downloadDirectory) {
        if (objectName.endsWith("/")) {
            return null;
        }
        final Path filePath = Paths.get(downloadDirectory.getAbsolutePath(), objectName);
        final File parentDirectory = filePath.getParent().toFile();
        if (!parentDirectory.exists() && !parentDirectory.mkdirs()) {
            throw new RuntimeException(String.format("Could not create %s in order to download %s", parentDirectory, objectName));
        }
        final File file = filePath.toFile();
        if (file.exists()) {
            return null;
        }

        return filePath;
    }

    protected static final String substringAfter(final String string, final String separator) {
        final int index = string.indexOf(separator);
        if (index < 0) {
            return "";
        }

        return string.substring(index + separator.length());
    }
}
