/*
 * (c) 2018-2019 Cloudera, Inc. All rights reserved.
 *
 *   This code is provided to you pursuant to your written agreement with Cloudera, which may be the terms of the
 *  Affero General Public License version 3 (AGPLv3), or pursuant to a written agreement with a third party authorized
 *  to distribute this code.  If you do not have a written agreement with Cloudera or with an authorized and
 *  properly licensed third party, you do not have any rights to this code.
 *
 *   If this code is provided to you under the terms of the AGPLv3:
 *   (A) CLOUDERA PROVIDES THIS CODE TO YOU WITHOUT WARRANTIES OF ANY KIND;
 *   (B) CLOUDERA DISCLAIMS ANY AND ALL EXPRESS AND IMPLIED WARRANTIES WITH RESPECT TO THIS CODE, INCLUDING BUT NOT
 *       LIMITED TO IMPLIED WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE;
 *   (C) CLOUDERA IS NOT LIABLE TO YOU, AND WILL NOT DEFEND, INDEMNIFY, OR HOLD YOU HARMLESS FOR ANY CLAIMS ARISING
 *       FROM OR RELATED TO THE CODE; AND
 *   (D) WITH RESPECT TO YOUR EXERCISE OF ANY RIGHTS GRANTED TO YOU FOR THE CODE, CLOUDERA IS NOT LIABLE FOR ANY
 *       DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED
 *       TO, DAMAGES RELATED TO LOST REVENUE, LOST PROFITS, LOSS OF INCOME, LOSS OF BUSINESS ADVANTAGE OR
 *       UNAVAILABILITY, OR LOSS OR CORRUPTION OF DATA.
 *
 */

package com.cloudera.naaf.metering;

import java.time.Instant;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

public class StandardInvocation implements Invocation {
    private final long durationNanos;
    private final boolean successful;
    private final String id;
    private final String functionId;
    private final String region;
    private final String functionName;
    private final CloudPlatform cloudPlatform;
    private final String flowCrn;
    private final Instant startTime;

    private StandardInvocation(final Builder builder) {
        this.id = UUID.randomUUID().toString();
        this.durationNanos = builder.durationNanos;
        this.successful = builder.successful;
        this.region = builder.region;
        this.flowCrn = builder.flowCrn;
        this.functionName = builder.functionName;
        this.cloudPlatform = builder.cloudPlatform;
        this.functionId = builder.functionId;
        this.startTime = builder.startTime;
    }

    @Override
    public long getDuration(final TimeUnit timeUnit) {
        return timeUnit.convert(durationNanos, TimeUnit.NANOSECONDS);
    }

    @Override
    public String getId() {
        return id;
    }

    @Override
    public String getFunctionId() {
        return functionId;
    }

    @Override
    public String getRegion() {
        return region;
    }

    @Override
    public CloudPlatform getCloudPlatform() {
        return cloudPlatform;
    }

    @Override
    public String getFunctionName() {
        return functionName;
    }

    @Override
    public String getFlowCrn() {
        return flowCrn;
    }

    @Override
    public Instant getStartTime() {
        return startTime;
    }

    @Override
    public boolean isSuccessful() {
        return successful;
    }

    public static Builder builder() {
        return new Builder();
    }

    public static class Builder {
        private long durationNanos;
        private boolean successful;
        private String region;
        private String functionName;
        private CloudPlatform cloudPlatform;
        private String flowCrn;
        private String functionId;
        private Instant startTime;

        private Builder() {
        }

        public Builder durationNanos(final long durationNanos) {
            this.durationNanos = durationNanos;
            return this;
        }

        public Builder successful(final boolean successful) {
            this.successful = successful;
            return this;
        }

        public Builder region(final String region) {
            this.region = region;
            return this;
        }

        public Builder functionName(final String functionName) {
            this.functionName = functionName;
            return this;
        }

        public Builder flowCrn(final String flowCrn) {
            this.flowCrn = flowCrn;
            return this;
        }

        public Builder cloudPlatform(final CloudPlatform cloudPlatform) {
            this.cloudPlatform = cloudPlatform;
            return this;
        }

        public Builder functionId(final String functionId) {
            this.functionId = functionId;
            return this;
        }

        public Builder startTime(final Instant startTime) {
            this.startTime = startTime;
            return this;
        }

        public StandardInvocation build() {
            return new StandardInvocation(this);
        }
    }
}
