/*
 * (c) 2018-2022 Cloudera, Inc. All rights reserved.
 *
 *   This code is provided to you pursuant to your written agreement with Cloudera, which may be the terms of the
 *  Affero General Public License version 3 (AGPLv3), or pursuant to a written agreement with a third party authorized
 *  to distribute this code.  If you do not have a written agreement with Cloudera or with an authorized and
 *  properly licensed third party, you do not have any rights to this code.
 *
 *   If this code is provided to you under the terms of the AGPLv3:
 *   (A) CLOUDERA PROVIDES THIS CODE TO YOU WITHOUT WARRANTIES OF ANY KIND;
 *   (B) CLOUDERA DISCLAIMS ANY AND ALL EXPRESS AND IMPLIED WARRANTIES WITH RESPECT TO THIS CODE, INCLUDING BUT NOT
 *       LIMITED TO IMPLIED WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE;
 *   (C) CLOUDERA IS NOT LIABLE TO YOU, AND WILL NOT DEFEND, INDEMNIFY, OR HOLD YOU HARMLESS FOR ANY CLAIMS ARISING
 *       FROM OR RELATED TO THE CODE; AND
 *   (D) WITH RESPECT TO YOUR EXERCISE OF ANY RIGHTS GRANTED TO YOU FOR THE CODE, CLOUDERA IS NOT LIABLE FOR ANY
 *       DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED
 *       TO, DAMAGES RELATED TO LOST REVENUE, LOST PROFITS, LOSS OF INCOME, LOSS OF BUSINESS ADVANTAGE OR
 *       UNAVAILABILITY, OR LOSS OR CORRUPTION OF DATA.
 *
 */

package com.cloudera.naaf.metering;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.TimeUnit;

public class NopMeteringPublisher implements MeteringPublisher {
    private static final Logger logger = LoggerFactory.getLogger(NopMeteringPublisher.class);

    private long millis;
    private int invocations;
    private int successful;

    @Override
    public void initialize() {
    }

    @Override
    public synchronized void publish(final Invocation invocation) {
        millis += roundDurationToMillis(invocation.getDuration(TimeUnit.NANOSECONDS));
        invocations++;

        if (invocation.isSuccessful()) {
            successful++;
        }

        flush();
    }

    private long roundDurationToMillis(final long nanos) {
        final long millis = nanos / 1_000_000;
        final long additionalNanos = nanos % (millis * 1_000_000);
        if (additionalNanos > 0) {
            return millis + 1;
        }

        return millis;
    }

    private synchronized void flush() {
        if (invocations == 0) {
            logger.debug("No invocations to report");
            return;
        }

        logger.info("Billing is disabled. Otherwise, billing notification would be sent for: {} invocations, {} of which were successful, for a total of {} milliseconds",
            invocations, successful, millis);

        reset();
    }

    private synchronized void reset() {
        invocations = 0;
        millis = 0;
        successful = 0;
    }
}
